/*
 * $Id$
 *
 * G N A P P L E T
 *
 * gnapplet is a gnbus protocol driver for symbian phones.
 *
 * This file is part of gnokii.
 *
 * Gnokii is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Gnokii is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with gnokii; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Copyright (C) 2004 BORBELY Zoltan
 *
 * This file contains the controlling logic of the application.
 *
 */

#include <e32cons.h>

#include "ircomm.h"
#include "rfcomm.h"
#include "pkt.h"
#include "proto.h"


static void HandleConnectionL(Device *dev)
{
	TBuf8<1024> b_in, b_out;
	PktBuf pkt_in, pkt_out;
	int type;

	do {
		b_in.Zero();
		b_out.Zero();
		ReadPacketL(dev, type, b_in);
		pkt_in.Set((void *)b_in.Ptr(), b_in.Length());
		pkt_out.Set(b_out.Ptr(), 1024);

		switch (type) {
		case GNAPPLET_MSG_INFO:
			HandleInfoMsg(pkt_in, pkt_out);
			break;
		case GNAPPLET_MSG_PHONEBOOK:
			HandlePhonebookMsg(pkt_in, pkt_out);
			break;
		}

		b_out.SetLength(pkt_out.GetOffs());
		WritePacketL(dev, type, b_out);
	} while (1);
}


static void MainL(CConsoleBase *cons)
{
	IrCOMM *ircomm;
	RfCOMM *rfcomm;
	RTimer timer;
	TRequestStatus ircomm_rs, rfcomm_rs, timer_rs;

	ircomm = new IrCOMM;
	CleanupStack::PushL(ircomm);
	ircomm->ConstructL();

	rfcomm = new RfCOMM;
	CleanupStack::PushL(rfcomm);
	rfcomm->ConstructL(14);

	ircomm->AcceptL(ircomm_rs);
	rfcomm->AcceptL(rfcomm_rs);
	timer.CreateLocal();
	timer.After(timer_rs, TTimeIntervalMicroSeconds32(30000 * 1000));

	while (timer_rs == KRequestPending) {
		User::WaitForAnyRequest();
		if (ircomm_rs == KErrNone) {
			cons->Printf(_L("IrCOMM connection\r\n"));
			HandleConnectionL(ircomm);
			break;
		}
		if (rfcomm_rs == KErrNone) {
			cons->Printf(_L("RfCOMM connection\r\n"));
			HandleConnectionL(rfcomm);
			break;
		}
	};

	CleanupStack::PopAndDestroy(rfcomm);
	CleanupStack::PopAndDestroy(ircomm);
}


static void SpawnMainL(void)
{
	CActiveScheduler *sched;
	CConsoleBase *cons;
	TInt error;

	error = StartC32();
	if (error != KErrNone && error != KErrAlreadyExists) {
		User::Leave(error);
	}

	sched = new (ELeave)CActiveScheduler;
	CleanupStack::PushL(sched);
	CActiveScheduler::Install(sched);

	cons = Console::NewL(_L("GNAPPLET"), TSize(KDefaultConsWidth, KDefaultConsHeight));
	CleanupStack::PushL(cons);

	cons->Printf(_L("gnapplet started\n"));
	MainL(cons);
	//cons->Printf(_L("[press any key]\n"));
	//cons->Getch();

	CleanupStack::PopAndDestroy(cons);

	CleanupStack::PopAndDestroy(sched);
}


GLDEF_C int E32Main(void)
{
	CTrapCleanup *cleanup = CTrapCleanup::New();

	TRAPD(error, SpawnMainL());
	if (error != KErrNone) {
		User::Panic(_L("GNAPPLET"), error);
	}

	delete cleanup;

	return 0;
}
