/*
 * $Id$
 *
 * G N A P P L E T
 *
 * gnapplet is a gnbus protocol driver for symbian phones.
 *
 * This file is part of gnokii.
 *
 * Gnokii is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Gnokii is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with gnokii; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Copyright (C) 2004 BORBELY Zoltan
 *
 * This file contains the phonebook message handler.
 *
 */

#include "proto.h"

#include <cntdef.h>
#include <cntdb.h>
#include <cntitem.h>
#include <cntfldst.h>


static CContactDatabase *cdb;
static CContactIdArray const *contacts;


static void RefreshL(void)
{
	if (cdb) return;

	cdb = CContactDatabase::OpenL();
	CleanupStack::PushL(cdb);

	contacts = cdb->SortedItemsL();
}


void HandlePhonebookMsg(PktBuf &in, PktBuf &out)
{
	TUint16 code, memtype, location, ns;
	CContactItem *item;
	TBuf<64> name;
	int i, n;

	in >> code;
	switch (code) {
	case GNAPPLET_MSG_PHONEBOOK_READ_REQ:
		in >> memtype;
		in >> location;
		in.FinishL();
		RefreshL();
		if (memtype != GN_MT_ME || location >= contacts->Count()) {
			out << (TUint16)GNAPPLET_MSG_PHONEBOOK_READ_RESP;
			out << (TUint16)GN_ERR_INVALIDLOCATION;
		} else {
			item = cdb->ReadContactL((*contacts)[location]);
			CleanupStack::PushL(item);
			CContactItemFieldSet &fieldset = item->CardFields();
			n = fieldset.Count();
			ns = 1;
			for (i = 0; i < n; i++) {
				const CContactItemField &field = fieldset[i];

				if (field.ContentType().ContainsFieldType(KUidContactFieldGivenName)) {
					name.Append(field.TextStorage()->Text());
				} else if (field.ContentType().ContainsFieldType(KUidContactFieldFamilyName)) {
					name.Append(field.TextStorage()->Text());
				} else if (field.ContentType().ContainsFieldType(KUidContactFieldPhoneNumber)) {
					ns++;
				} else if (field.ContentType().ContainsFieldType(KUidContactFieldEMail)) {
					ns++;
				}
			}
			out << (TUint16)GNAPPLET_MSG_PHONEBOOK_READ_RESP;
			out << (TUint16)GN_ERR_NONE;
			out << ns;
			out << (TUint16)GN_PHONEBOOK_ENTRY_Name << name;
			for (i = 0; i < n; i++) {
				const CContactItemField &field = fieldset[i];

				if (field.ContentType().ContainsFieldType(KUidContactFieldPhoneNumber)) {
					out << (TUint16)GN_PHONEBOOK_ENTRY_Number << field.TextStorage()->Text();
					ns++;
				} else if (field.ContentType().ContainsFieldType(KUidContactFieldEMail)) {
					out << (TUint16)GN_PHONEBOOK_ENTRY_Email << field.TextStorage()->Text();
				}
			}
			CleanupStack::PopAndDestroy(item);
		}
		break;

	default:
		out << (TUint16)(code | 1);
		out << (TUint16)GN_ERR_NOTSUPPORTED;
		break;
	}
}
